package com.yiring.common.aspect;

import com.yiring.common.constant.DateFormatter;
import com.yiring.common.core.Result;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

/**
 * 请求接口切面，记录接口耗时
 *
 * @author ifzm
 * @version 0.1
 */

@Aspect
@Component
public class RequestAspect {

    @Pointcut("@annotation(org.springframework.web.bind.annotation.PostMapping) || @annotation(org.springframework.web.bind.annotation.GetMapping) || @annotation(org.springframework.web.bind.annotation.ExceptionHandler)")
    public void apiPointCut() {
    }

    @Around("apiPointCut()")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        long start = System.currentTimeMillis();
        Object result = point.proceed();
        long end = System.currentTimeMillis();

        if (result instanceof Result) {
            ((Result<?>) result).setTimestamp(LocalDateTime.now().format(DateFormatter.DATE_TIME));
            ((Result<?>) result).setTimes(String.format("%.3fs", (double) (end - start) / 1000));
        }

        return result;
    }

}
